package oop6_dat6_u;

/**
 *
 * @author Felix Rohrer <felix.rohrer@stud.hslu.ch>
 */
public class Cube implements Comparable<Cube>
{

    private int number; //Nummer des Kubus
    private int s1, s2, s3; //Die 3 Seiten des Kubus

    public Cube(int no, int a, int b, int c)
    {
        this.number = no;
        this.s1 = a;
        this.s2 = b;
        this.s3 = c;
    }

    /**
     * Berechnet das Volumen des Kubus
     * @return Volumen
     */
    public int getVolume()
    {
        return (s1 * s2 * s3);
    }

    /**
     * Berechnet die Oberfläche des Kubus
     * @return Oberfläche
     */
    public int getSurface()
    {
        return (2 * (s1 * s2 + s1 * s3 + s2 * s3));
    }

    /**
     * Liefert die grösste Seitenlänge als Ergebnis
     * @return grösse Seitenlänge
     */
    public int getMaxDimension()
    {
        int res;
        if (s1 > s2) {
            if (s1 > s3) {
                res = s1;
            } else {
                res = s3;
            }
        } else {
            if (s2 > s3) {
                res = s2;
            } else {
                res = s3;
            }
        }
        return res;
    }

    /**
     * String des Kubus
     * @return Beschreibung des Kubus
     */
    @Override
    public String toString()
    {
        return ("---------------------------------------\n"+
            "CUBE " + number + "\n" +
            "Sides: " + s1 + " " + s2 + " " + s3 + "\n" +
            "Volume = " + this.getVolume() + "\n" +
            "Surface = " + this.getSurface()+ "\n");
    }

    /**
     * Kubus vergleichen
     * @param other Kubus
     * @return Kubus gleich oder nicht
     */
    @Override
    public boolean equals(Object other)
    {
        if (this == other) // 1. Test auf Identität
        {
            return true;
        }
        if (other == null) // 2. Test auf null
        {
            return false;
        }
        if (other.getClass() != this.getClass())// 3. Test auf Vergleichbarkeit
        {
            return false;
        }
        if ( ! (this.getVolume() == ((Cube) other).getVolume()))// 4. Vergleich relev. Felder
        {
            return false;
        }
        return true;
    }

    /**
     * Hash-Code wird bassierend auf der (eindeutigen) Nummer definiert
     * @return Hash-Code
     */
    @Override
    public int hashCode()
    {
        return number;
    }

    /**
     * Kubus wird auf ihre Volumen verglichen
     * @param c Cube zum Vergleichen
     * @return Differenz des Volumen
     */
    public int compareTo(Cube c)
    {
        return (this.getVolume() - c.getVolume());
    }


    /**
     * kleine "test" Methode
     * @param args
     */public static void main(String[] args)
    {
        Cube c1 = new Cube(1, 1, 2, 3);
        Cube c2 = new Cube(2, 10, 20, 30);
        Cube c3 = new Cube(3, 30, 20, 10);

        System.out.println(c1.toString());
        System.out.println(c2.toString());
        System.out.println(c3.toString());

        System.out.println("c1 == c2 ? : " + c1.equals(c2));
        System.out.println("c2 == c3 ? : " + c2.equals(c3));
        System.out.println("c1 > c2 ? : " + c1.compareTo(c2));
        System.out.println("c2 > c3 ? : " + c2.compareTo(c3));
    }
}
